.header { font-size:11pt; font-weight:bold; color:#000; background-color:rgba(255,255,255,0.45); }
.date { font-size:10pt; font-weight:bold; color:#000; background-color:rgba(255,255,255,0.45); }
.line-0 { font-size:10pt; color:#000; background-color:rgba(255,255,255,0.25); }
.line-1 { font-size:10pt; color:#000; background-color:transparent; }
.time { font-size:7pt; color:#555; padding-top:3px; }
.title, .description { }
.description { font-size:9pt; }
.text { width:100%; }
.title_with_description { font-weight:bold; }
a { text-decoration:none; color:#222; }
table { width:100%; border-spacing:0px; }
td { vertical-align: top; }
body { margin:0px; background-color:transparent }
